# Reseda version 1.0.0
# ResedaBlock.py
# by Antoine Rauzy
# Copytargets (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Nodes
# 4) Edges
# 5) Graphs

# 1) Imported modules
# -------------------

from S2MLCore import *
from ResedaType import *

# 2) Creator
# ----------

class ResedaBlock(S2MLBlock):
  def __init__(self, path, container):
    S2MLBlock.__init__(self, path, container)
    self._nodes = []
    self._edges = []

# 3) Nodes
# --------

  def LookForNode(self, name):
    element = self.LookForIdentifiedElement(name)
    if element==None or not ResedaType.IsNode(element):
      return None
    return element

  def AccessNode(self, path):
    element = self.AccessNamedElement(path)
    if element==None or not ResedaType.IsNode(element):
      return None
    return element

  def GetNumberOfNodes(self):
    return len(self._nodes)

  def GetNodes(self):
    return self._nodes

  def GetNode(self, index):
    if index<0 and index>=self.GetNumberOfNodes():
      return None
    return self._nodes[index]

  def AppendNode(self, node):
    node.SetIndex(self.GetNumberOfNodes())
    self._nodes.append(node)

# 4) Edges
# --------

  def LookForEdge(self, name):
    element = self.LookForIdentifiedElement(name)
    if element==None or not ResedaType.IsEdge(element):
      return None
    return element

  def AccessEdge(self, path):
    element = self.AccessNamedElement(path)
    if element==None or (not ResedaType.IsEdge(element) and not ResedaType.IsArc(element)):
      return None
    return element

  def GetNumberOfEdges(self):
    return len(self._edges)

  def GetEdges(self):
    return self._edges

  def GetEdge(self, index):
    if index<0 and index>=self.GetNumberOfEdges():
      return None
    return self._edges[index]

  def AppendEdge(self, edge):
    self._edges.append(edge)

# 5) Graphs
# ---------

  def BuildGraph(self):
    self._nodes = []
    self._edges = []
    self._CollectNodes(self)
    self._CollectEdges(self)

  def _CollectNodes(self, block):
    for element in block._elements:
      if element.GetType()==ResedaType.BLOCK:
        self._CollectNodes(element)
      elif element.GetType()==ResedaType.NODE:
        element.ResetInEdges()
        element.ResetOutEdges()
        self.AppendNode(element)

  def _CollectEdges(self, block):
    for element in block._elements:
      if element.GetType()==ResedaType.BLOCK:
        self._CollectEdges(element)
      elif element.GetType()==ResedaType.EDGE:
        sourceNode = element.GetSourceNode()
        sourceNode.AddOutEdge(element)
        targetNode = element.GetTargetNode()
        targetNode.AddInEdge(element)
        self.AppendEdge(element)
      elif element.GetType()==ResedaType.ARC:
        sourceNode = element.GetSourceNode()
        sourceNode.AddOutEdge(element)
        sourceNode.AddInEdge(element)
        targetNode = element.GetTargetNode()
        targetNode.AddInEdge(element)
        targetNode.AddOutEdge(element)
        self.AppendEdge(element)
