# Reseda version 1.0.0
# ResedaEdge.py
# by Antoine Rauzy
# Copytargets (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Accessors

# 1) Imported modules
# -------------------

from S2MLCore import *
from ResedaType import *

# 2) Creator
# ----------

class ResedaEdge(S2MLElement):
  def __init__(self, type, path, sourceNodePath, targetNodePath, container):
    S2MLElement.__init__(self, type, container)
    self.SetPath(path)
    if path.IsIdentifier():
      self.SetIdentifierAndInsertInContainerDictionary(path.ToString())
    self._sourceNodePath = sourceNodePath
    self._sourceNode = None
    self._targetNodePath = targetNodePath
    self._targetNode = None

  @staticmethod
  def NewEdge(path, sourceNodePath, targetNodePath, container):
    edge = ResedaEdge(ResedaType.EDGE, path, sourceNodePath, targetNodePath, container)
    return edge

  @staticmethod
  def NewArc(path, sourceNodePath, targetNodePath, container):
    arc = ResedaEdge(ResedaType.ARC, path, sourceNodePath, targetNodePath, container)
    return arc

# 3) Accessors
# ------------

  def IsEdge(self):
    return self.GetType()==ResedaType.EDGE

  def IsArc(self):
    return self.GetType()==ResedaType.ARC

  def GetSourceNodePath(self):
    return self._sourceNodePath

  def SetSourceNodePath(self, path):
   self._sourceNodePath = path

  def GetSourceNode(self):
    return self._sourceNode

  def SetSourceNode(self, node):
    self._sourceNode = node

  def GetTargetNodePath(self):
    return self._targetNodePath

  def SetTargetNodePath(self, path):
   self._targetNodePath = path

  def GetTargetNode(self):
    return self._targetNode

  def SetTargetNode(self, node):
    self._targetNode = node

  def GetOppositeNode(self, node):
    if node==self._sourceNode:
      return self._targetNode
    if node==self._targetNode:
      return self._sourceNode
    return None
