# Reseda version 1.0.0
# ResedaModel.py
# by Antoine Rauzy
# Copytargets (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creator
# 3) Methods to test types
# 4) Factory of elements
# 5) Methods for the parser

# 1) Imported modules
# -------------------

from S2MLCore import *

from ResedaType import *
from ResedaNode import *
from ResedaEdge import *
from ResedaVariable import *
from ResedaSetExpression import *
from ResedaBlock import *

# 2) Creator
# ----------

class ResedaModel(S2MLModel):

  def __init__(self):
    S2MLModel.__init__(self)

# 3) Methods to test types
# ------------------------

  @staticmethod
  def IsNode(element):
    return element.GetType()==ResedaType.NODE

  @staticmethod
  def IsEdge(element):
    return element.GetType()==ResedaType.EDGE

  @staticmethod
  def IsArc(element):
    return element.GetType()==ResedaType.ARC

# 4) Factory of elements
# ----------------------

  def NewBlock(self, path, container):
    return ResedaBlock(path, container)

  def NewNode(self, path, container):
    return ResedaNode(path, container)

  def NewEdge(self, path, sourcePorthPath, targetPorthPath, container):
    return ResedaEdge.NewEdge(path, sourcePorthPath, targetPorthPath, container)

  def NewArc(self, path, sourcePorthPath, targetPorthPath, container):
    return ResedaEdge.NewArc(path, sourcePorthPath, targetPorthPath, container)

  def NewVariable(self, path, container):
    return ResedaVariable(path, container)

# 5) Methods for the parser
# -------------------------

  def CanComposeNode(self, container):
    return True

  def CanComposeEdge(self, container):
    return True

